(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./BinaryTreeNode"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const BinaryTreeNode_1 = require("./BinaryTreeNode");
    class BinaryTree {
        constructor(comparator = (a, b) => a > b) {
            this.root = null;
            this.comparator = comparator;
        }
        add(value) {
            let currentNode = this.root;
            let futureParent = null;
            const newNode = new BinaryTreeNode_1.BinaryTreeNode(value, null);
            while (currentNode !== null) {
                futureParent = currentNode;
                if (futureParent !== null) {
                    if (this.comparator(futureParent.value, value)) {
                        currentNode = futureParent.left;
                    }
                    else {
                        currentNode = futureParent.right;
                    }
                }
            }
            newNode.parent = futureParent;
            if (futureParent === null) {
                this.root = newNode;
            }
            else if (this.comparator(futureParent.value, value)) {
                futureParent.left = newNode;
            }
            else {
                futureParent.right = newNode;
            }
        }
        remove(value) {
            let node = this.search(value);
            if (node === null) {
                return false;
            }
            if (!node.hasLeftChild()) {
                this.transplant(node, node.right);
            }
            else if (!node.hasRightChild()) {
                this.transplant(node, node.left);
            }
            else {
                let successor = node.right.min();
                if (successor !== node.right) {
                    this.transplant(successor, successor.right);
                    successor.right = node.right;
                    successor.right.parent = successor;
                }
                this.transplant(node, successor);
            }
        }
        inOrderTreeWalk(callback, initialValue) {
            return this.inorderNodeWalk(this.root, callback, initialValue);
        }
        isEmpty() {
            return this.root === null;
        }
        max() {
            const currentNode = this.root;
            if (this.isEmpty()) {
                return null;
            }
            return currentNode.max();
        }
        min() {
            const currentNode = this.root;
            if (this.isEmpty()) {
                return null;
            }
            return currentNode.min();
        }
        reverseTreeWalk(callback, initialValue) {
            return this.reverseNodeWalk(this.root, callback, initialValue);
        }
        search(value) {
            let currentNode = this.root;
            while (currentNode && currentNode.value !== value) {
                if (this.comparator(currentNode.value, value)) {
                    currentNode = currentNode.left;
                }
                else {
                    currentNode = currentNode.right;
                }
            }
            return currentNode;
        }
        successor(value) {
            let node = this.search(value);
            let ancestor;
            if (node === null || value === null) {
                return null;
            }
            if (node.right !== null) {
                return node.right.min();
            }
            ancestor = node.parent;
            while (ancestor !== null && ancestor.right === node) {
                node = ancestor;
                ancestor = node.parent;
            }
            return ancestor;
        }
        inorderNodeWalk(node, callback, previousValue) {
            if (node !== null) {
                previousValue = this.inorderNodeWalk(node.left, callback, previousValue);
                previousValue = callback(previousValue, node.value);
                previousValue = this.inorderNodeWalk(node.right, callback, previousValue);
                return previousValue;
            }
            else {
                return previousValue;
            }
        }
        reverseNodeWalk(node, callback, previousValue) {
            if (node !== null) {
                previousValue = this.reverseNodeWalk(node.right, callback, previousValue);
                previousValue = callback(previousValue, node.value);
                previousValue = this.reverseNodeWalk(node.left, callback, previousValue);
                return previousValue;
            }
            else {
                return previousValue;
            }
        }
        transplant(node, newNode) {
            if (node.isRoot()) {
                this.root = newNode;
            }
            else if (node.isLeftChild()) {
                node.parent.left = newNode;
            }
            else {
                node.parent.right = newNode;
            }
            if (newNode !== null) {
                newNode.parent = node.parent;
            }
        }
    }
    exports.BinaryTree = BinaryTree;
});
//# sourceMappingURL=BinaryTree.js.map